<?php

namespace Pinpoint\CarouselManager\Block;

use Magento\Catalog\Model\Template\Filter\Factory;
use Magento\Cms\Model\BlockFactory;
use Magento\Cms\Model\Template\FilterProvider;
use Magento\Framework\View\Element\Template;
use Magento\Framework\View\Element\Template\Context;
use Magento\Widget\Block\BlockInterface;
use Pinpoint\CarouselManager\Model\CarouselRepository;

/**
 * Class Carousel
 * @package Pinpoint\CarouselManager\Block
 */
class Carousel extends Template
{
    /**
     * @var
     */
    protected $carouselId;

    /**
     * @var CMS Block ID
     */
    protected $cmsBlockId;

    /**
     * @var CarouselRepository
     */
    protected $carouselRespository;
    /**
     * @var
     */
    protected $carousel;

    /**
     * @var string
     */
    protected $_template;

    /**
     * @var string
     */
    protected $templateFilterModel = 'Magento\Catalog\Model\Template\Filter';
    /**
     * @var
     */
    protected $templateProcessor;
    /**
     * @var Factory
     */
    protected $templateFilterFactory;

    /**
     * @var array
     */
    protected $slides = [];

    /**
     * @var FilterProvider
     */
    protected $_filterProvider;

    /**
     * @var BlockFactory
     */
    protected $_blockFactory;

    /**
     * Carousel constructor.
     * @param CarouselRepository $carouselRepository
     * @param Factory $templateFilterFactory
     * @param FilterProvider $filterProvider
     * @param BlockFactory $blockFactory
     * @param Context $context
     * @param array $data
     */
    public function __construct(
        CarouselRepository $carouselRepository,
        Factory $templateFilterFactory,
        FilterProvider $filterProvider,
        BlockFactory $blockFactory,
        Context $context,
        array $data = []
    )
    {
        $this->templateFilterFactory = $templateFilterFactory;
        $this->carouselRespository = $carouselRepository;

        $this->_template = 'Pinpoint_CarouselManager::carousel.phtml';

        if (isset($data['template']) && $data['template']) {
            $this->setTemplate($data['template']);
        }

        if (isset($data['carousel_id']) && $data['carousel_id']) {
            $this->setCarouselId($data['carousel_id']);
        }

        if (isset($data['cms_block_id']) && $data['cms_block_id']) {
            $this->setCmsBlockId($data['cms_block_id']);
        }

        parent::__construct($context, $data);

        $this->_filterProvider = $filterProvider;
        $this->_blockFactory = $blockFactory;
    }

    /**
     * @return mixed
     */
    public function getCarouselId()
    {
        return $this->carouselId;
    }

    /**
     * @param $carouselId
     */
    public function setCarouselId($carouselId)
    {
        $this->carouselId = $carouselId;
    }

    /**
     * @return mixed
     */
    public function getCmsBlockId()
    {
        return $this->cmsBlockId;
    }

    /**
     * @param $cmsBlockId
     */
    public function setCmsBlockId($cmsBlockId)
    {
        $this->cmsBlockId = $cmsBlockId;
    }

    /**
     * @return mixed
     */
    public function getCurrentStore()
    {
        return $this->_storeManager->getStore()->getId();
    }

    /**
     * @return mixed|null
     */
    public function getCarousel()
    {
        if (!$this->carousel) {

            try {

                $carouselId = $this->getCarouselId();
                $carousel = $this->carouselRespository->getById($carouselId);
                $currentStoreId = $this->getCurrentStore();

            } catch (\Exception $e) {
                // Silently skip
                $this->_logger->critical($e);
                return null;
            }

            $allowStores = $carousel->getStoreIds();

            if ($allowStores && $currentStoreId && is_array($allowStores) && count($allowStores) && in_array($currentStoreId, $allowStores)) {
                $this->carousel = $carousel;
            }

        }

        return $this->carousel;
    }

    /**
     * @return string
     */
    public function getCMSBlock()
    {

        if ($identifier = $this->getCmsBlockId()) {

            $storeId = $this->_storeManager->getStore()->getId();

            /** @var \Magento\Cms\Model\Block $block */
            $block = $this->_blockFactory->create();
            $block->setStoreId($storeId)->load($identifier);

            $html = $this->_filterProvider->getBlockFilter()->setStoreId($storeId)->filter($block->getContent());
            return $html;
        }

        return '';

    }

    /**
     * @return string
     */
    protected function _toHtml()
    {
        if (!$this->getCarousel()) {
            return '';
        }
        return parent::_toHtml();
    }

    /**
     * @return mixed
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    protected function _getTemplateProcessor()
    {
        if (null === $this->templateProcessor) {
            $this->templateProcessor = $this->templateFilterFactory->create($this->templateFilterModel);
        }

        return $this->templateProcessor;
    }

    /**
     * @param $attribute
     * @return bool
     */
    public function isFilterable($attribute)
    {
        return in_array($attribute, ['title', 'description', 'cta', 'link']);
    }

    /**
     * @param $slide
     * @param $attribute
     * @param string $default
     * @return mixed
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getSlideProp($slide, $attribute, $default = '')
    {
        if (!isset($this->slides[$slide->getId()][$attribute])) {
            $val = $slide->getData($attribute) ? $slide->getData($attribute) : $default;
            if ($this->isFilterable($attribute) && $val) {
                $val = $this->_getTemplateProcessor()->filter($val);
            }

            $this->slides[$slide->getId()][$attribute] = $val;
        }
        return $this->slides[$slide->getId()][$attribute];
    }

    /**
     * @param $slide
     * @return mixed
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getImageUrl($slide)
    {
        if (!isset($this->slides[$slide->getId()]['image'])) {
            $this->slides[$slide->getId()]['image'] = null;
            $imagePath = $this->getSlideProp($slide, 'image', null);
            if ($imagePath) {
                $imageUrl = $this->getBaseUrl() . $imagePath;
                if ($imageUrl) {
                    $this->slides[$slide->getId()]['image'] = $imageUrl;
                }
            }

        }
        return $this->slides[$slide->getId()]['image'];
    }

    /**
     * @param $slide
     * @return mixed
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getMobileImageUrl($slide)
    {
        if (!isset($this->slides[$slide->getId()]['mobile_image'])) {
            $this->slides[$slide->getId()]['mobile_image'] = null;
            $imagePath = $this->getSlideProp($slide, 'mobile_image', null);
            if ($imagePath) {
                $imageUrl = $this->getBaseUrl() . $imagePath;
                if ($imageUrl) {
                    $this->slides[$slide->getId()]['mobile_image'] = $imageUrl;
                }
            }

        }
        return $this->slides[$slide->getId()]['mobile_image'];
    }


}
